package cn.com.qmth.examcloud.core.oe.student.api.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.student.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.student.bean.CheckExamInProgressInfo;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamProcessResultInfo;
import cn.com.qmth.examcloud.core.oe.student.bean.StartExamInfo;
import cn.com.qmth.examcloud.core.oe.student.service.ExamControlService;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.enums.HandInExamType;
import cn.com.qmth.examcloud.support.examing.ExamingSession;
import cn.com.qmth.examcloud.web.helpers.SequenceLockHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@Api(tags = "在线考试控制")
@RestController
@RequestMapping("${app.api.oe.student}/examControl")
public class ExamControlController extends ControllerSupport {

    @Autowired
    private ExamControlService examControlService;

    /**
     * 开始考试
     */
    @ApiOperation(value = "开始考试")
    @GetMapping("/startExam")
    public StartExamInfo startExam(@RequestParam Long examStudentId) {
        User user = getAccessUser();
        String sequenceLockKey = Constants.START_EXAM_LOCK_PREFIX + user.getUserId();
        StartExamInfo startExamInfo;
        // 开始考试上锁,分布式锁,系统在请求结束后会，自动释放锁，无需手动解锁
        SequenceLockHelper.getLock(sequenceLockKey);
        Check.isNull(examStudentId, "examStudentId不能为空");

        startExamInfo = examControlService.startExam(examStudentId, user);
        return startExamInfo;
    }

    /**
     * 断点续考:检查正在进行中的考试
     */
    @ApiOperation(value = "断点续考:检查正在进行中的考试")
    @GetMapping("/checkExamInProgress")
    public ExamProcessResultInfo checkExamInProgress() {
        User user = getAccessUser();
        String sequenceLockKey = Constants.START_EXAM_LOCK_PREFIX + user.getUserId();
        // 系统在请求结束后会，自动释放锁，无需手动解锁
        SequenceLockHelper.getLock(sequenceLockKey);
        ExamProcessResultInfo res = new ExamProcessResultInfo();
        try {
            CheckExamInProgressInfo info = examControlService.checkExamInProgress(user.getUserId());
            res.setCode(Constants.COMMON_SUCCESS_CODE);
            res.setData(info);
            return res;
        } catch (StatusException e) {
            if (e.getCode().equals(Constants.EXAM_RECORD_NOT_END_STATUS_CODE)) {
                res.setCode(Constants.PROCESSING_EXAM_RECORD_CODE);
                return res;
            }
            throw e;
        } catch (Exception e) {
            throw e;
        }
    }

    /**
     * 结束考试：交卷..
     */
    @ApiOperation(value = "结束考试：交卷")
    @GetMapping("/endExam")
    public void endExam() {
        User user = getAccessUser();
        Long studentId = user.getUserId();
        String sequenceLockKey = Constants.END_EXAM_LOCK_PREFIX + studentId;
        //系统在请求结束后会，自动释放锁，无需手动解锁
        SequenceLockHelper.getLock(sequenceLockKey);

        long st = System.currentTimeMillis();
        long startTime = System.currentTimeMillis();

        ExamingSession examingSession = null;
//        // 获取考试会话，判断考生是否已结束考试
//        ExamSessionInfo examSessionInfo = examSessionInfoService.getExamSessionInfo(studentId);
//        if (examSessionInfo == null) {
//            throw new StatusException("oestudent-100100", "考试会话已过期");
//        }


        if (log.isDebugEnabled()) {
            log.debug("0 [END_EXAM] 交卷前处理耗时:" + (System.currentTimeMillis() - startTime) + " ms");
        }
        examControlService.handInExam(examingSession.getExamRecordDataId(), HandInExamType.MANUAL);
        if (log.isDebugEnabled()) {
            log.debug("1 [END_EXAM]合计 耗时:" + (System.currentTimeMillis() - st) + " ms");
        }
    }

}
