package cn.com.qmth.examcloud.core.oe.student.dao.entity;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Table;

import cn.com.qmth.examcloud.support.enums.IsSuccess;
import org.hibernate.annotations.DynamicInsert;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.support.enums.HandInExamType;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.web.jpa.JpaEntity;

/**
 * @author chenken
 * @date 2018年8月13日 上午11:41:34
 * @company QMTH
 * @description 考试记录数据表, 与ec_oe_exam_record一对一关系
 */
@Entity
@Table(name = "ec_oes_exam_record_data", indexes = {
        @Index(name = "IDX_E_O_E_R_D_001", columnList = "examStudentId"),
        @Index(name = "IDX_E_O_E_R_D_002", columnList = "studentId"),
        @Index(name = "IDX_E_O_E_R_D_003", columnList = "examId"),
        @Index(name = "IDX_E_O_E_R_D_004", columnList = "courseId"),
})
@DynamicInsert
public class ExamRecordDataEntity extends JpaEntity {
    /**
     *
     */
    private static final long serialVersionUID = -242327915801750970L;
    /**
     * 主键
     */
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;
    /**
     * 考试ID
     */
    private Long examId;
    /**
     * 考试类型
     */
    @Column(length = 20)
    @Enumerated(EnumType.STRING)
    private ExamType examType;
    /**
     * 考生ID
     */
    private Long examStudentId;
    /**
     * 学生ID
     */
    private Long studentId;
    /**
     * 课程ID
     */
    private Long courseId;

    /**
     * 学习中心ID
     */
    private Long orgId;
    /**
     * 顶级机构ID
     */
    private Long rootOrgId;
    /**
     * 基础试卷ID
     */
    private String basePaperId;

    /**
     * 试卷类型
     */
    private String paperType;

    /**
     * 考试开始时间
     */
    private Date startTime;
    /**
     * 考试结束时间
     */
    private Date endTime;

    /**
     * 考试时长
     */
    private Long usedExamTime;
    /**
     * 是否断点续考
     */
    private Boolean isContinued;
    /**
     * 断点续考次数
     */
    private Integer continuedCount;
    /**
     * 是否达到最大断点限制
     */
    private Boolean isExceed;
    /**
     * 抓拍比对成功次数
     */
    private Integer faceSuccessCount;
    /**
     * 抓拍比对失败次数
     */
    private Integer faceFailedCount;
    /**
     * 抓拍存在陌生人的次数
     */
    private Integer faceStrangerCount;
    /**
     * 抓拍比对总次数
     */
    private Integer faceTotalCount;

    /**
     * 考试记录状态
     */
    private ExamRecordStatus examRecordStatus;

    /**
     * 交卷类型
     */
    private HandInExamType handInExamType;

    /**
     *
     * 活体检测结果
     */
    @Column(length = 20)
    @Enumerated(EnumType.STRING)
    private IsSuccess faceVerifyResult;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public ExamType getExamType() {
        return examType;
    }

    public void setExamType(ExamType examType) {
        this.examType = examType;
    }

    public Long getExamStudentId() {
        return examStudentId;
    }

    public void setExamStudentId(Long examStudentId) {
        this.examStudentId = examStudentId;
    }

    public Long getStudentId() {
        return studentId;
    }

    public void setStudentId(Long studentId) {
        this.studentId = studentId;
    }

    public Long getCourseId() {
        return courseId;
    }

    public void setCourseId(Long courseId) {
        this.courseId = courseId;
    }

    public Long getOrgId() {
        return orgId;
    }

    public void setOrgId(Long orgId) {
        this.orgId = orgId;
    }

    public Long getRootOrgId() {
        return rootOrgId;
    }

    public void setRootOrgId(Long rootOrgId) {
        this.rootOrgId = rootOrgId;
    }

    public String getBasePaperId() {
        return basePaperId;
    }

    public void setBasePaperId(String basePaperId) {
        this.basePaperId = basePaperId;
    }

    public String getPaperType() {
        return paperType;
    }

    public void setPaperType(String paperType) {
        this.paperType = paperType;
    }

    public Date getStartTime() {
        return startTime;
    }

    public void setStartTime(Date startTime) {
        this.startTime = startTime;
    }

    public Date getEndTime() {
        return endTime;
    }

    public void setEndTime(Date endTime) {
        this.endTime = endTime;
    }

    public Long getUsedExamTime() {
        return usedExamTime;
    }

    public void setUsedExamTime(Long usedExamTime) {
        this.usedExamTime = usedExamTime;
    }

    public Boolean getIsContinued() {
        return isContinued;
    }

    public void setIsContinued(Boolean isContinued) {
        this.isContinued = isContinued;
    }

    public Integer getContinuedCount() {
        return continuedCount;
    }

    public void setContinuedCount(Integer continuedCount) {
        this.continuedCount = continuedCount;
    }

    public Integer getFaceSuccessCount() {
        return faceSuccessCount;
    }

    public void setFaceSuccessCount(Integer faceSuccessCount) {
        this.faceSuccessCount = faceSuccessCount;
    }

    public Integer getFaceFailedCount() {
        return faceFailedCount;
    }

    public void setFaceFailedCount(Integer faceFailedCount) {
        this.faceFailedCount = faceFailedCount;
    }

    public Integer getFaceStrangerCount() {
        return faceStrangerCount;
    }

    public void setFaceStrangerCount(Integer faceStrangerCount) {
        this.faceStrangerCount = faceStrangerCount;
    }

    public Integer getFaceTotalCount() {
        return faceTotalCount;
    }

    public void setFaceTotalCount(Integer faceTotalCount) {
        this.faceTotalCount = faceTotalCount;
    }

    public Boolean getIsExceed() {
        return isExceed;
    }

    public void setIsExceed(Boolean isExceed) {
        this.isExceed = isExceed;
    }

    public ExamRecordStatus getExamRecordStatus() {
        return examRecordStatus;
    }

    public void setExamRecordStatus(ExamRecordStatus examRecordStatus) {
        this.examRecordStatus = examRecordStatus;
    }

    public HandInExamType getHandInExamType() {
        return handInExamType;
    }

    public void setHandInExamType(HandInExamType handInExamType) {
        this.handInExamType = handInExamType;
    }

    public IsSuccess getFaceVerifyResult() {
        return faceVerifyResult;
    }

    public void setFaceVerifyResult(IsSuccess faceVerifyResult) {
        this.faceVerifyResult = faceVerifyResult;
    }
}