package cn.com.qmth.examcloud.core.oe.student.service.impl;

import java.util.Date;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordDataService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamBean;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.support.examing.ExamingSession;
import cn.com.qmth.examcloud.web.redis.RedisClient;

/**
 * @author chenken
 * @date 2018/8/15 11:16
 * @company QMTH
 * @description 考试记录数据服务实现
 */
@Service("examRecordDataService")
public class ExamRecordDataServiceImpl implements ExamRecordDataService {

    @Autowired
    private RedisClient redisClient;

    @Value("${exam_record_data_key_prefix}")
    private String examRecordDataKeyPrefix;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Transactional
    @Override
    public ExamRecordData createExamRecordData(ExamingSession examingSession, ExamBean examBean,
            CourseBean courseBean, String basePaperId) {
        ExamRecordDataEntity examRecordData = new ExamRecordDataEntity();
        examRecordData.setExamId(examBean.getId());
        examRecordData.setExamType(ExamType.valueOf(examBean.getExamType()));

        examRecordData.setExamStudentId(examingSession.getExamStudentId());
        examRecordData.setStudentId(examingSession.getStudentId());
        examRecordData.setOrgId(examingSession.getOrgId());
        examRecordData.setRootOrgId(examingSession.getRootOrgId());

        examRecordData.setCourseId(courseBean.getId());
        examRecordData.setBasePaperId(basePaperId);

        examRecordData.setPaperType(examingSession.getPaperType());

        examRecordData.setStartTime(new Date());
        examRecordData.setIsContinued(false);
        examRecordData.setContinuedCount(0);
        examRecordData.setFaceSuccessCount(0);
        examRecordData.setFaceTotalCount(0);
        examRecordData.setFaceFailedCount(0);
        examRecordData.setFaceStrangerCount(0);
        examRecordData.setExamRecordStatus(ExamRecordStatus.EXAM_ING);
        examRecordDataRepo.save(examRecordData);
        ExamRecordData bean = new ExamRecordData();
        BeanUtils.copyProperties(bean, examRecordData);
        //存入redis
        saveExamRecordDataCache(examRecordData.getId(), bean);
        return bean;
    }

    @Override
    public void saveExamRecordDataCache(Long examRecordDataId, ExamRecordData data) {
        redisClient.set(examRecordDataKeyPrefix + examRecordDataId, data, -1);
    }

    @Override
    public ExamRecordData getExamRecordDataCache(Long examRecordDataId) {
        return redisClient.get(examRecordDataKeyPrefix + examRecordDataId, ExamRecordData.class);
    }

    @Override
    public void deleteExamRecordDataCache(Long examRecordDataId) {
        redisClient.delete(examRecordDataKeyPrefix + examRecordDataId);
    }

}
