package cn.com.qmth.examcloud.core.oe.student.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.core.oe.student.base.bean.ExamQuestion;
import cn.com.qmth.examcloud.core.oe.student.base.bean.ExamRecordQuestions;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultPaper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionGroup;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionStructureWrapper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionUnitWrapper;
import cn.com.qmth.examcloud.web.redis.RedisClient;

@Service("examRecordQuestionsService")
public class ExamRecordQuestionsServiceImpl implements ExamRecordQuestionsService {

    @Autowired
    private RedisClient redisClient;

    @Value("${exam_record_questions_key_prefix}")
    private String examRecordQuestionsKeyPrefix;

    @Override
    public void saveExamRecordQuestions(Long examRecordDataId, ExamRecordQuestions questions) {
        redisClient.set(examRecordQuestionsKeyPrefix + examRecordDataId, questions,-1);
    }

    @Override
    public ExamRecordQuestions getExamRecordQuestions(Long examRecordDataId) {
        return redisClient.get(examRecordQuestionsKeyPrefix + examRecordDataId, ExamRecordQuestions.class);
    }

    @Override
    public void deleteExamRecordQuestions(Long examRecordDataId) {
        redisClient.delete(examRecordQuestionsKeyPrefix + examRecordDataId);
    }

    @Override
    public ExamRecordQuestions createExamRecordQuestions(Long examRecordDataId, DefaultPaper defaultPaper) {
        ExamRecordQuestions examRecordQuestions = new ExamRecordQuestions();

        List<ExamQuestion> examQuestionEntityList = new ArrayList<ExamQuestion>();
        List<DefaultQuestionGroup> defaultQuestionGroups = defaultPaper.getQuestionGroupList();
        int order = 0;
        for (int i = 0; i < defaultQuestionGroups.size(); i++) {
            DefaultQuestionGroup defaultQuestionGroup = defaultQuestionGroups.get(i);
            List<DefaultQuestionStructureWrapper> defaultQuestionStructureWrappers = defaultQuestionGroup
                    .getQuestionWrapperList();
            for (DefaultQuestionStructureWrapper defaultQuestionStructureWrapper : defaultQuestionStructureWrappers) {
                List<DefaultQuestionUnitWrapper> questionUnitWrapperList = defaultQuestionStructureWrapper
                        .getQuestionUnitWrapperList();
                for (DefaultQuestionUnitWrapper defaultQuestionUnitWrapper : questionUnitWrapperList) {
                    ExamQuestion examQuestionEntity = new ExamQuestion();
                    examQuestionEntity.setExamRecordDataId(examRecordDataId);
                    examQuestionEntity.setMainNumber(i + 1);
                    examQuestionEntity.setOrder(++order);
                    examQuestionEntity.setQuestionId(defaultQuestionStructureWrapper.getQuestionId());
                    examQuestionEntity.setQuestionScore(defaultQuestionUnitWrapper.getQuestionScore());
                    examQuestionEntity.setQuestionType(defaultQuestionUnitWrapper.getQuestionType());
                    examQuestionEntity.setOptionPermutation(defaultQuestionUnitWrapper.getOptionPermutation());
                    examQuestionEntity.setAudioPlayTimes(null);
                    examQuestionEntity.setAnswerType(defaultQuestionUnitWrapper.getAnswerType());

                    examQuestionEntityList.add(examQuestionEntity);
                }
            }
        }

        examRecordQuestions.setExamQuestions(examQuestionEntityList);
        examRecordQuestions.setExamRecordDataId(examRecordDataId);
        examRecordQuestions.setCreationTime(new Date());
        saveExamRecordQuestions(examRecordDataId, examRecordQuestions);
        return examRecordQuestions;
    }

}
